#property copyright ""
#property link      ""

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color3 Blue
#property indicator_color4 Red
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 1
#property indicator_width4 1
#property strict

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_tbiased2,   // Trend biased (extreme) price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased,  // Heiken ashi trend biased price
   pr_hatbiased2  // Heiken ashi trend biased (extreme) price
};

extern int      Length           = 7;                // Price Channel Period
extern double   Risk             = 3;                // Risk Factor in units (0...10) 
extern bool     UseReEntry       = true;             // Re-Entry?
extern enPrices Price            = pr_close;         // Price
extern bool     alertsOn         = true;             // Turn alerts on?
extern bool     alertsOnCurrent  = false;            // Alerts on current (still opened) bar?
extern bool     alertsMessage    = true;             // Alerts showing a pop-up message?
extern bool     alertsNotify     = false;            // Alerts sending a push notification?
extern bool     alertsSound      = false;            // Alerts playing sound?
extern bool     alertsEmail      = false;            // Alerts sending email?
extern string   soundFile        = "alert2.wav";     // Alerts Sound File to use

//
//
//
//
//

double UpSignal[];
double DnSignal[]; 
double UpEntry[];
double DnEntry[]; 
double smax[];
double smin[];
double trend[];

//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,UpSignal); SetIndexStyle(0,DRAW_ARROW); SetIndexArrow(0,108); SetIndexLabel(0,"UpSignal");
   SetIndexBuffer(1,DnSignal); SetIndexStyle(1,DRAW_ARROW); SetIndexArrow(1,108); SetIndexLabel(1,"DnSignal");
   SetIndexBuffer(2,UpEntry);  SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,241); SetIndexLabel(2,"UpRe-Entry");
   SetIndexBuffer(3,DnEntry);  SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,242); SetIndexLabel(3,"DnRe-Entry");
   SetIndexBuffer(4,smax);
   SetIndexBuffer(5,smin);
   SetIndexBuffer(6,trend);
   IndicatorShortName("PriceChannel_Signal ("+(string)Length+")");
return(0);
}

//
//
//
//
//

int start()
{
   
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = fmin(Bars-counted_bars,Bars-1);
   
   //
   //
   //
   //
   //
         
   for(i=limit; i>=0; i--)
   {   
      if (i<Bars-1)
      {
        double price  = getPrice(Price,Open,Close,High,Low,i);
        double price1 = getPrice(Price,Open,Close,High,Low,i+1);  
        double hi     = High[iHighest(NULL,0,MODE_HIGH,Length,i)];   
        double lo     = Low[iLowest(NULL,  0,MODE_LOW, Length,i)]; 
        smax[i] = hi - (33-Risk)*(hi-lo)/100;
        smin[i] = lo + (33-Risk)*(hi-lo)/100;
        DnSignal[i] = EMPTY_VALUE;
        DnEntry[i]  = EMPTY_VALUE;
        UpEntry[i]  = EMPTY_VALUE;
        UpSignal[i] = EMPTY_VALUE;
        trend[i] = (i<Bars-1) ? (price>smax[i]) ? 1 : (price<smin[i]) ? -1 : trend[i+1] : 0;
   
	     //
	     //
	     //
	     //
        //
	    
        if (trend[i]>0)
	     {
	        if (trend[i+1]<0)                                                                       UpSignal[i] = Low[i]-0.5*iATR(NULL,0,Length,i);
	        if (UseReEntry && price > smax[i] && price1 <= smax[i+1] && UpSignal[i] == EMPTY_VALUE) UpEntry[i]  = Low[i]-0.5*iATR(NULL,0,Length,i); 
	      }
	      if (trend[i]<0)
	      {
            if (trend[i+1]>0)                                                                        DnSignal[i] = High[i]+0.5*iATR(NULL,0,Length,i);
	         if (UseReEntry && price < smin[i] && price1 >= smin[i+1] && DnSignal[i] == EMPTY_VALUE)  DnEntry[i]  = High[i]+0.5*iATR(NULL,0,Length,i); 
	       }
	       
	   }	         
	}
	
	//
	//
	//
	//
	//
	
	if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (UpSignal[whichBar] != EMPTY_VALUE && UpSignal[whichBar+1] == EMPTY_VALUE) doAlert(whichBar,"up");
      if (DnSignal[whichBar] != EMPTY_VALUE && DnSignal[whichBar+1] == EMPTY_VALUE) doAlert(whichBar,"down");
      if (UpEntry[whichBar]  != EMPTY_VALUE && UpEntry[whichBar+1]  == EMPTY_VALUE) doAlert(whichBar,"Re-entry for buy");
      if (DnEntry[whichBar]  != EMPTY_VALUE && DnEntry[whichBar+1]  == EMPTY_VALUE) doAlert(whichBar,"Re-entry for sell");       
    } 	         
return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

#define _priceInstances     1
#define _priceInstancesSize 4
double workHa[][_priceInstances*_priceInstancesSize];
double getPrice(int price, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (price>=pr_haclose)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars); int r = Bars-i-1; instanceNo*=_priceInstancesSize;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = MathMax(high[i], MathMax(haOpen,haClose));
         double haLow   = MathMin(low[i] , MathMin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (price)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
            case pr_hatbiased2:
               if (haClose>haOpen)  return(haHigh);
               if (haClose<haOpen)  return(haLow);
                                    return(haClose);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
      case pr_tbiased2:   
               if (close[i]>open[i]) return(high[i]);
               if (close[i]<open[i]) return(low[i]);
                                     return(close[i]);        
   }                                     
   return(0);
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," PriceChannel Signal ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " PriceChannel Signal "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

  

